@@ -27,6 +27,7 @@ package global;
 
 import com.sun.source.util.JavacTask;
 import com.sun.tools.javac.Main;
+import com.sun.tools.javac.api.JavacTaskImpl;
 import global.ap1.AP;
 import global.ap1.ClassBasedAP;
 import global.ap1.ErrorProducingAP;
@@ -40,12 +41,15 @@ import java.util.Arrays;
 import java.util.Collection;
 import java.util.LinkedList;
 import java.util.List;
+import javax.lang.model.element.Element;
+import javax.lang.model.element.TypeElement;
 import javax.tools.Diagnostic;
 import javax.tools.DiagnosticCollector;
 import javax.tools.JavaCompiler;
 import javax.tools.JavaFileObject;
 import javax.tools.SimpleJavaFileObject;
 import javax.tools.ToolProvider;
+import static junit.framework.Assert.assertEquals;
 import junit.framework.TestCase;
 
 /**
@@ -244,4 +248,19 @@ public class AnnotationProcessingTest extends TestCase {
             this.content = content;
         }
     }
+
+    public void testKeepBrokenAttributes228628() throws IOException {
+        final String bootPath = System.getProperty("sun.boot.class.path"); //NOI18N
+        final JavaCompiler tool = ToolProvider.getSystemJavaCompiler();
+        assert tool != null;
+
+        String code = "package test; @Test(NonExistent.class) public @interface Test { public Class<?> value(); }";
+
+        JavacTaskImpl ct = (JavacTaskImpl)tool.getTask(null, null, null, Arrays.asList("-bootclasspath",  bootPath, "-Xjcov", "-XDshouldStopPolicy=GENERATE"), null, Arrays.asList(new global.Test116436.MyFileObject(code)));
+        Iterable<? extends Element> classes = ct.enter();
+        TypeElement test = (TypeElement) classes.iterator().next();
+        
+        assertEquals(1, test.getAnnotationMirrors().get(0).getElementValues().size());
+    }
+    
 }
