@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2005, 2013, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -44,6 +44,7 @@ import com.sun.tools.javac.code.Symbol.*;
 import com.sun.tools.javac.comp.*;
 import com.sun.tools.javac.file.JavacFileManager;
 import com.sun.tools.javac.main.*;
+import com.sun.tools.javac.main.JavaCompiler;
 import com.sun.tools.javac.model.*;
 import com.sun.tools.javac.parser.JavacParser;
 import com.sun.tools.javac.parser.Parser;
@@ -52,7 +53,6 @@ import com.sun.tools.javac.tree.*;
 import com.sun.tools.javac.tree.JCTree.*;
 import com.sun.tools.javac.util.*;
 import com.sun.tools.javac.util.List;
-import com.sun.tools.javac.main.JavaCompiler;
 
 /**
  * Provides access to functionality specific to the JDK Java Compiler, javac.
@@ -65,29 +65,26 @@ import com.sun.tools.javac.main.JavaCompiler;
  * @author Peter von der Ah&eacute;
  * @author Jonathan Gibbons
  */
-public class JavacTaskImpl extends JavacTask {
-    private ClientCodeWrapper ccw;
+public class JavacTaskImpl extends BasicJavacTask {
     private Main compilerMain;
     private JavaCompiler compiler;
     private Locale locale;
     private String[] args;
     private String[] classNames;
-    private Context context;
     private List<JavaFileObject> fileObjects;
     private Map<JavaFileObject, JCCompilationUnit> notYetEntered;
     private ListBuffer<Env<AttrContext>> genList;
-    private TaskListener taskListener;
-    private AtomicBoolean used = new AtomicBoolean();
+    private final AtomicBoolean used = new AtomicBoolean();
     private Iterable<? extends Processor> processors;
-    
-    private Integer result = null;
+
+    private Main.Result result = null;
 
     JavacTaskImpl(Main compilerMain,
                 String[] args,
                 String[] classNames,
                 Context context,
                 List<JavaFileObject> fileObjects) {
-        this.ccw = ClientCodeWrapper.instance(context);
+        super(null, false);
         this.compilerMain = compilerMain;
         this.args = args;
         this.classNames = classNames;
@@ -101,18 +98,19 @@ public class JavacTaskImpl extends JavacTask {
     }
 
     JavacTaskImpl(Main compilerMain,
-                Iterable<String> flags,
+                Iterable<String> args,
                 Context context,
                 Iterable<String> classes,
                 Iterable<? extends JavaFileObject> fileObjects) {
-        this(compilerMain, toArray(flags), toArray(classes), context, toList(fileObjects));
+        this(compilerMain, toArray(args), toArray(classes), context, toList(fileObjects));
     }
 
     JavacTaskImpl(Context context) {
+        super(null, false);
         this.context = context;
         setLocale(Locale.getDefault());
     }
-
+    
     static private String[] toArray(Iterable<String> iter) {
         ListBuffer<String> result = new ListBuffer<String>();
         if (iter != null)
@@ -130,19 +128,23 @@ public class JavacTaskImpl extends JavacTask {
         return result.toList();
     }
 
-    public Boolean call() {
+    public Main.Result doCall() {
         if (!used.getAndSet(true)) {
             initContext();
             notYetEntered = new HashMap<JavaFileObject, JCCompilationUnit>();
             compilerMain.setAPIMode(true);
             result = compilerMain.compile(args, classNames, context, fileObjects, processors);
             cleanup();
-            return result == 0;
+            return result;
         } else {
             throw new IllegalStateException("multiple calls to method 'call'");
         }
     }
 
+    public Boolean call() {
+        return doCall().isOK();
+    }
+
     public void setProcessors(Iterable<? extends Processor> processors) {
         processors.getClass(); // null check
         // not mt-safe
@@ -167,10 +169,11 @@ public class JavacTaskImpl extends JavacTask {
                 throw new IllegalStateException();
         } else {
             initContext();
+            compilerMain.log = Log.instance(context);
             compilerMain.setOptions(Options.instance(context));
             compilerMain.filenames = new LinkedHashSet<File>();
             Collection<File> filenames = compilerMain.processArgs(CommandLine.parse(args), classNames);
-            if (!filenames.isEmpty())
+            if (filenames != null && !filenames.isEmpty())
                 throw new IllegalArgumentException("Malformed arguments " + toString(filenames, " "));
             compiler = JavaCompiler.instance(context);
             // NOTE: this value will be updated after annotation processing
@@ -199,11 +202,7 @@ public class JavacTaskImpl extends JavacTask {
     }
 
     private void initContext() {
-        context.put(JavacTaskImpl.class, this);
-        if (context.get(TaskListener.class) != null)
-            context.put(TaskListener.class, (TaskListener)null);
-        if (taskListener != null)
-            context.put(TaskListener.class, ccw.wrap(taskListener));
+        context.put(JavacTask.class, this);
         //initialize compiler's default locale
         context.put(Locale.class, locale);
     }
@@ -233,10 +232,6 @@ public class JavacTaskImpl extends JavacTask {
         return fm.getRegularFile(file);
     }
 
-    public void setTaskListener(TaskListener taskListener) {
-        this.taskListener = taskListener;
-    }
-
     public Iterable<? extends CompilationUnitTree> parse (JavaFileObject... files) throws IOException {
         prepareCompiler();
         java.util.List<CompilationUnitTree> trees = new java.util.LinkedList<CompilationUnitTree> ();
@@ -326,8 +321,11 @@ public class JavacTaskImpl extends JavacTask {
     public Iterable<? extends TypeElement> enter(Iterable<? extends CompilationUnitTree> trees)
         throws IOException
     {
-        prepareCompiler();
+        if (trees == null && notYetEntered != null && notYetEntered.isEmpty())
+            return List.nil();
 
+        prepareCompiler();
+        
         ListBuffer<JCCompilationUnit> roots = null;
 
         if (trees == null) {
@@ -365,6 +363,9 @@ public class JavacTaskImpl extends JavacTask {
         if (roots == null)
             return List.nil();
 
+        if (!compiler.skipAnnotationProcessing && compiler.processAnnotations && compiler.deferredDiagnosticHandler == null)
+            compiler.deferredDiagnosticHandler = new Log.DeferredDiagnosticHandler(compiler.log);
+
         try {
             List<JCCompilationUnit> units = compiler.enterTrees(roots.toList());
 
@@ -385,7 +386,7 @@ public class JavacTaskImpl extends JavacTask {
                         elements.append(unit.packge.package_info);
                 } else {
                     for (JCTree node : unit.defs) {
-                        if (node.getTag() == JCTree.CLASSDEF) {
+                        if (node.hasTag(JCTree.Tag.CLASSDEF)) {
                             JCClassDecl cdef = (JCClassDecl) node;
                             if (cdef.sym != null) // maybe null if errors in anno processing
                                 elements.append(cdef.sym);
@@ -428,7 +429,7 @@ public class JavacTaskImpl extends JavacTask {
         final java.util.List<TypeElement> res = new java.util.ArrayList<TypeElement>();
         if (((JCCompilationUnit)tree).packge != null) {
             for (JCTree t : ((JCCompilationUnit)tree).defs) {
-                if (t.getTag() == JCTree.CLASSDEF) {
+                if (t.hasTag(JCTree.Tag.CLASSDEF)) {
                     ClassSymbol sym = ((JCClassDecl)t).sym;
                     if (sym != null)
                         res.add(sym);
@@ -485,12 +486,12 @@ public class JavacTaskImpl extends JavacTask {
         private void handleFlowResults(Queue<Env<AttrContext>> queue, ListBuffer<Element> elems) {
             for (Env<AttrContext> env: queue) {
                 switch (env.tree.getTag()) {
-                    case JCTree.CLASSDEF:
+                    case CLASSDEF:
                         JCClassDecl cdef = (JCClassDecl) env.tree;
                         if (cdef.sym != null)
                             elems.append(cdef.sym);
                         break;
-                    case JCTree.TOPLEVEL:
+                    case TOPLEVEL:
                         JCCompilationUnit unit = (JCCompilationUnit) env.tree;
                         if (unit.packge != null)
                             elems.append(unit.packge);
@@ -630,22 +631,6 @@ public class JavacTaskImpl extends JavacTask {
         abstract void process(Env<AttrContext> env);
     }
 
-    /**
-     * For internal use only.  This method will be
-     * removed without warning.
-     */
-    public Context getContext() {
-        return context;
-    }
-
-    /**
-     * For internal use only.  This method will be
-     * removed without warning.
-     */
-    public void updateContext(Context newContext) {
-        context = newContext;
-    }
-
     /**
      * For internal use only.  This method will be
      * removed without warning.
@@ -655,17 +640,16 @@ public class JavacTaskImpl extends JavacTask {
             throw new IllegalArgumentException();
         compiler = JavaCompiler.instance(context);
         JavaFileObject prev = compiler.log.useSource(null);
-        boolean old = compiler.log.suppressErrorsAndWarnings;
-        compiler.log.suppressErrorsAndWarnings = true;
         ParserFactory parserFactory = ParserFactory.instance(context);
         Attr attr = Attr.instance(context);
+        Log.DiagnosticHandler discardHandler = new Log.DiscardDiagnosticHandler(compiler.log);
         try {
             CharBuffer buf = CharBuffer.wrap((expr+"\u0000").toCharArray(), 0, expr.length());
-            Parser parser = parserFactory.newParser(buf, false, false, false, true);
+            Parser parser = parserFactory.newParser(buf, false, false, false);
             JCTree tree = parser.parseType();
             return attr.attribType(tree, (Symbol.TypeSymbol)scope);
         } finally {
-            compiler.log.suppressErrorsAndWarnings = old;
+            compiler.log.popDiagnosticHandler(discardHandler);
             compiler.log.useSource(prev);
         }
     }
@@ -675,16 +659,14 @@ public class JavacTaskImpl extends JavacTask {
             throw new IllegalArgumentException();
         compiler = JavaCompiler.instance(context);
         JavaFileObject prev = compiler.log.useSource(null);
-        boolean old = compiler.log.suppressErrorsAndWarnings;
-        compiler.log.suppressErrorsAndWarnings = true;
         ParserFactory parserFactory = ParserFactory.instance(context);
-        Attr attr = Attr.instance(context);
+        Log.DiagnosticHandler discardHandler = new Log.DiscardDiagnosticHandler(compiler.log);
         try {
             CharBuffer buf = CharBuffer.wrap((expr+"\u0000").toCharArray(), 0, expr.length());
-            Parser parser = parserFactory.newParser(buf, false, false, false, true);
+            Parser parser = parserFactory.newParser(buf, false, false, false);
             return parser.parseType();
         } finally {
-            compiler.log.suppressErrorsAndWarnings = old;
+            compiler.log.popDiagnosticHandler(discardHandler);
             compiler.log.useSource(prev);
         }
     }
@@ -693,21 +675,21 @@ public class JavacTaskImpl extends JavacTask {
         return (JCStatement) doParse(ParseKind.STATEMENT, stmt, pos, errors);
     }
 
-    private JCTree doParse(ParseKind kind, CharSequence source, SourcePositions[] pos, DiagnosticListener<? super JavaFileObject> errors) {
+    private JCTree doParse(ParseKind kind, CharSequence source, SourcePositions[] pos, final DiagnosticListener<? super JavaFileObject> errors) {
         if (source == null || (pos != null && pos.length != 1))
             throw new IllegalArgumentException();
         compiler = JavaCompiler.instance(context);
         JavaFileObject prev = compiler.log.useSource(null);
-        boolean old = compiler.log.suppressErrorsAndWarnings;
-        DiagnosticListener<? super JavaFileObject> oldListener = compiler.log.getDiagnosticListener();
-        int oldErrors = compiler.log.nerrors;
-        int oldWarnings = compiler.log.nwarnings;
-        compiler.log.suppressErrorsAndWarnings = false;
-        compiler.log.setDiagnosticListener(errors);
+        Log.DiagnosticHandler discardHandler = new Log.DiscardDiagnosticHandler(compiler.log) {
+            @Override
+            public void report(JCDiagnostic diag) {
+                errors.report(diag);
+            }            
+        };
         ParserFactory parserFactory = ParserFactory.instance(context);
         try {
             CharBuffer buf = CharBuffer.wrap((source+"\u0000").toCharArray(), 0, source.length());
-            Parser parser = parserFactory.newParser(buf, false, true, false, true);
+            Parser parser = parserFactory.newParser(buf, false, true, false);
             if (parser instanceof JavacParser) {
                 if (pos != null)
                     pos[0] = new ParserSourcePositions((JavacParser)parser);
@@ -717,18 +699,15 @@ public class JavacTaskImpl extends JavacTask {
                     case VARIABLE_INIT: return ((JavacParser)parser).variableInitializer();
                     case STATIC_BLOCK:
                         List<JCTree> trees = ((JavacParser)parser).classOrInterfaceBodyDeclaration(null, false);
-                        return trees.head != null && trees.head.getTag() == JCTree.BLOCK ? (JCBlock) trees.head : null;
+                        return trees.head != null && trees.head.hasTag(JCTree.Tag.BLOCK) ? (JCBlock) trees.head : null;
                     default: throw new UnsupportedOperationException(kind.name());
                 }
                 
             }
             return null;
         } finally {
-            compiler.log.suppressErrorsAndWarnings = old;
-            compiler.log.setDiagnosticListener(oldListener);
+            compiler.log.popDiagnosticHandler(discardHandler);
             compiler.log.useSource(prev);
-            compiler.log.nerrors = oldErrors;
-            compiler.log.nwarnings = oldWarnings;
         }
     }
 
@@ -774,14 +753,16 @@ public class JavacTaskImpl extends JavacTask {
         Log log = Log.instance(context);
         Attr attr = Attr.instance(context);
         JavaFileObject prev = log.useSource(null);
-        boolean old = log.suppressErrorsAndWarnings;
-        log.suppressErrorsAndWarnings = true;
+        Log.DiagnosticHandler discardHandler = new Log.DiscardDiagnosticHandler(log);
+        Enter enter = Enter.instance(context);
+        enter.shadowTypeEnvs(true);
         try {
             if (tree instanceof JCExpression)
                 return attr.attribExpr(tree, env, Type.noType);
             return attr.attribStat(tree, env);
         } finally {
-            log.suppressErrorsAndWarnings = old;
+            enter.shadowTypeEnvs(false);
+            log.popDiagnosticHandler(discardHandler);
             log.useSource(prev);
         }
     }
@@ -790,13 +771,15 @@ public class JavacTaskImpl extends JavacTask {
         Log log = Log.instance(context);
         Attr attr = Attr.instance(context);
         JavaFileObject prev = log.useSource(null);
-        boolean old = log.suppressErrorsAndWarnings;
-        log.suppressErrorsAndWarnings = true;
+        Log.DiagnosticHandler discardHandler = new Log.DiscardDiagnosticHandler(log);
+        Enter enter = Enter.instance(context);
+        enter.shadowTypeEnvs(true);
         try {
             Env<AttrContext> ret = tree instanceof JCExpression ? attr.attribExprToTree(tree, env, to) : attr.attribStatToTree(tree, env, to);
             return new JavacScope(ret);
         } finally {
-            log.suppressErrorsAndWarnings = old;
+            enter.shadowTypeEnvs(false);
+            log.popDiagnosticHandler(discardHandler);
             log.useSource(prev);
         }
     }
