@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2011, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2013, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -40,7 +40,7 @@ import com.sun.tools.javac.code.Scope.*;
 import com.sun.tools.javac.code.Symbol.*;
 import com.sun.tools.javac.code.Type.*;
 import com.sun.tools.javac.jvm.*;
-import com.sun.tools.javac.main.RecognizedOptions.PkgInfo;
+import com.sun.tools.javac.main.Option.PkgInfo;
 import com.sun.tools.javac.model.LazyTreeLoader;
 import com.sun.tools.javac.tree.*;
 import com.sun.tools.javac.tree.JCTree.*;
@@ -56,7 +56,7 @@ import static com.sun.tools.javac.code.Kinds.*;
  *  the symbol table. The pass consists of two phases, organized as
  *  follows:
  *
- *  <p>In the first phase, all class symbols are intered into their
+ *  <p>In the first phase, all class symbols are entered into their
  *  enclosing scope, descending recursively down the tree for classes
  *  which are members of other classes. The class symbols are given a
  *  MemberEnter object as completer.
@@ -84,12 +84,12 @@ import static com.sun.tools.javac.code.Kinds.*;
  *
  *  <p>Classes migrate from one phase to the next via queues:
  *
- *  <pre>
+ *  <pre>{@literal
  *  class enter -> (Enter.uncompleted)         --> member enter (1)
  *              -> (MemberEnter.halfcompleted) --> member enter (2)
  *              -> (Todo)                      --> attribute
  *                                              (only for toplevel classes)
- *  </pre>
+ *  }</pre>
  *
  *  <p><b>This is NOT part of any supported API.
  *  If you write code that depends on this, you do so at your own risk.
@@ -144,7 +144,11 @@ public class Enter extends JCTree.Visitor {
 
         predefClassDef = make.ClassDef(
             make.Modifiers(PUBLIC),
-            syms.predefClass.name, null, null, null, null);
+            syms.predefClass.name,
+            List.<JCTypeParameter>nil(),
+            null,
+            List.<JCExpression>nil(),
+            List.<JCTree>nil());
         predefClassDef.sym = syms.predefClass;
         todo = Todo.instance(context);
         fileManager = context.get(JavaFileManager.class);
@@ -182,7 +186,7 @@ public class Enter extends JCTree.Visitor {
         Env<AttrContext> lintEnv = localEnv;
         while (lintEnv.info.lint == null)
             lintEnv = lintEnv.next;
-        localEnv.info.lint = lintEnv.info.lint.augment(sym.attributes_field, sym.flags());
+        localEnv.info.lint = lintEnv.info.lint.augment(sym);
         return localEnv;
     }
 
@@ -253,7 +257,7 @@ public class Enter extends JCTree.Visitor {
      *  only, and members go into the class member scope.
      */
     Scope enterScope(Env<AttrContext> env) {
-        return (env.tree.getTag() == JCTree.CLASSDEF)
+        return (env.tree.hasTag(JCTree.Tag.CLASSDEF))
             ? ((JCClassDecl) env.tree).sym.members_field
             : env.info.scope;
     }
@@ -353,7 +357,7 @@ public class Enter extends JCTree.Visitor {
                                 tree.packge);
                     if (addEnv || (tree0.packageAnnotations.isEmpty() &&
                                    tree.docComments != null &&
-                                   tree.docComments.get(tree) != null)) {
+                                   tree.docComments.hasComment(tree))) {
                         typeEnvs.put(tree.packge, topEnv);
                     }
                 }
@@ -373,6 +377,8 @@ public class Enter extends JCTree.Visitor {
         compilationUnits.put(tree.sourcefile.toUri(), tree);
         classEnter(tree.defs, topEnv);
         if (addEnv) {
+            if ((tree.packge.flags_field & APT_CLEANED) != 0)
+                todo.remove(tree.packge);
             todo.append(topEnv);
         }
         log.useSource(prev);
@@ -504,21 +510,28 @@ public class Enter extends JCTree.Visitor {
         } else if (reattr && c.completer == null) {
             new ElementScanner6<Void, Void>() {
                 @Override
-                public Void visitType(TypeElement e, Void p) {
-                    if (e instanceof ClassSymbol)
-                        ((ClassSymbol) e).flags_field |= FROMCLASS;
-                    return super.visitType(e, p);
-                }
-                @Override
-                public Void visitExecutable(ExecutableElement e, Void p) {
-                    if (e instanceof MethodSymbol)
-                        ((MethodSymbol) e).flags_field |= FROMCLASS;
+                public Void visitType(TypeElement te, Void p) {
+                    if (te instanceof ClassSymbol && ((ClassSymbol) te).completer == null) {
+                        ((ClassSymbol) te).flags_field |= FROMCLASS;
+                        for (Scope.Entry e = ((ClassSymbol) te).members().elems; e != null; e = e.sibling) {
+                            try {
+                                if (e.sym != null && e.sym.owner == te)
+                                    scan(e.sym);
+                            } catch (CompletionFailure cf) {}
+                        }
+                    }
                     return null;
                 }
                 @Override
-                public Void visitVariable(VariableElement e, Void p) {
-                    if (e instanceof VarSymbol)
-                        ((VarSymbol) e).flags_field |= FROMCLASS;
+                public Void visitExecutable(ExecutableElement ee, Void p) {
+                    if (ee instanceof MethodSymbol)
+                        ((MethodSymbol) ee).flags_field |= FROMCLASS;
+                    return null;
+                }
+                @Override
+                public Void visitVariable(VariableElement ve, Void p) {
+                    if (ve instanceof VarSymbol)
+                        ((VarSymbol) ve).flags_field |= FROMCLASS;
                     return null;
                 }
             }.scan(c);
