@@ -34,10 +34,10 @@ import com.sun.tools.javac.code.Symbol.MethodSymbol;
 import com.sun.tools.javac.code.Symtab;
 import com.sun.tools.javac.code.Type;
 import com.sun.tools.javac.code.Type.ClassType;
-import com.sun.tools.javac.code.TypeTags;
+import com.sun.tools.javac.code.TypeTag;
 import com.sun.tools.javac.code.Types;
 import com.sun.tools.javac.jvm.Pool;
-import com.sun.tools.javac.parser.Token;
+import com.sun.tools.javac.parser.Tokens;
 import com.sun.tools.javac.tree.JCTree;
 import com.sun.tools.javac.tree.JCTree.JCAssignOp;
 import com.sun.tools.javac.tree.JCTree.JCBinary;
@@ -46,6 +46,7 @@ import com.sun.tools.javac.tree.JCTree.JCCase;
 import com.sun.tools.javac.tree.JCTree.JCClassDecl;
 import com.sun.tools.javac.tree.JCTree.JCErroneous;
 import com.sun.tools.javac.tree.JCTree.JCExpression;
+import com.sun.tools.javac.tree.JCTree.JCExpressionStatement;
 import com.sun.tools.javac.tree.JCTree.JCImport;
 import com.sun.tools.javac.tree.JCTree.JCLiteral;
 import com.sun.tools.javac.tree.JCTree.JCMethodDecl;
@@ -71,6 +72,7 @@ import java.util.logging.Logger;
 import javax.lang.model.element.ElementKind;
 import javax.tools.JavaFileObject;
 
+
 /**
  *
  * @author Dusan Balek
@@ -99,6 +101,7 @@ public class Repair extends TreeTranslator {
     private TreeMaker make;
     private JCDiagnostic.Factory diags;
     private boolean allowEnums;
+    private boolean allowLambda;
     
     private Env<AttrContext> attrEnv;
     private boolean hasError;
@@ -123,6 +126,7 @@ public class Repair extends TreeTranslator {
         diags = JCDiagnostic.Factory.instance(context);
         Source source = Source.instance(context);
         allowEnums = source.allowEnums();
+        allowLambda = source.allowLambda();
     }
 
     @Override
@@ -144,7 +148,7 @@ public class Repair extends TreeTranslator {
         }
         if (tree.type != null && tree.type.isErroneous() || tree.type == syms.unknownType) {
             JCTree parent = parents.head;
-            if (parent == null || parent.getTag() != JCTree.CLASSDEF) {
+            if (parent == null || !parent.hasTag(JCTree.Tag.CLASSDEF)) {
                 hasError = true;
                 if (err == null && errMessage == null)
                     errMessage = "Erroneous tree type: " + tree.type;
@@ -152,18 +156,18 @@ public class Repair extends TreeTranslator {
         }
         if (!(hasError && tree instanceof JCStatement))
             return tree;
-        if (tree.getTag() == JCTree.CASE)
+        if (tree.hasTag(JCTree.Tag.CASE))
             return tree;
-        if (tree.getTag() == JCTree.CLASSDEF || tree.getTag() == JCTree.VARDEF) {
+        if (tree.hasTag(JCTree.Tag.CLASSDEF) || tree.hasTag(JCTree.Tag.VARDEF)) {
             JCTree parent = parents.head;
-            if (parent == null || (parent.getTag() != JCTree.BLOCK && parent.getTag() != JCTree.CASE))
+            if (parent == null || (!parent.hasTag(JCTree.Tag.BLOCK) && !parent.hasTag(JCTree.Tag.CASE)))
                 return tree;
         }
         String msg = err != null ? err.getMessage(null) : errMessage;
         hasError = false;
         err = null;
         errMessage = null;
-        if (tree.getTag() == JCTree.BLOCK) {
+        if (tree.hasTag(JCTree.Tag.BLOCK)) {
             ((JCBlock)tree).stats = List.of(generateErrStat(tree.pos(), msg));
             return tree;
         }
@@ -182,8 +186,8 @@ public class Repair extends TreeTranslator {
     @Override
     public void visitTypeParameter(JCTypeParameter tree) {
         super.visitTypeParameter(tree);
-        if (tree.type != null && tree.type.tag == TypeTags.TYPEVAR) {
-            Type.TypeVar tv = (Type.TypeVar)tree.type;
+        if (tree.type != null && tree.type.hasTag(TypeTag.TYPEVAR)) {
+            Type.TypeVar tv = (Type.TypeVar)tree.type.unannotatedType();
             if (tv.bound != null && tv.bound.isErroneous() || tv.bound == syms.unknownType) {
                 if (err == null && errMessage == null)
                     errMessage = "Erroneous type var bound: " + tv.bound;
@@ -208,7 +212,7 @@ public class Repair extends TreeTranslator {
         }
         if (hasError) {
             JCTree parent = parents != null ? parents.tail.head : null;
-            if (parent != null && parent.getTag() == JCTree.CLASSDEF) {
+            if (parent != null && parent.hasTag(JCTree.Tag.CLASSDEF)) {
                 tree.init = err != null ? generateErrExpr(err.getTree(), err.getMessage(null)) : generateErrExpr(tree.init, errMessage);
                 hasError = false;
                 err = null;
@@ -218,7 +222,7 @@ public class Repair extends TreeTranslator {
             }
         } else if (tree.sym == null) {
             JCTree parent = parents != null ? parents.tail.head : null;
-            if (parent != null && parent.getTag() != JCTree.CLASSDEF) {
+            if (parent != null && !parent.hasTag(JCTree.Tag.CLASSDEF)) {
                 hasError = true;
                 if (err == null && errMessage == null)
                     errMessage = "Null tree sym: " + tree;
@@ -240,9 +244,7 @@ public class Repair extends TreeTranslator {
         if (isErrClass && tree.body != null) {
             JCMethodInvocation app = TreeInfo.firstConstructorCall(tree);
             Name meth = app != null ? TreeInfo.name(app.meth) : null;
-            Symbol sym = app != null ? TreeInfo.symbol(app.meth) : null;
-            if (meth != null && (meth == meth.table.names._this || meth == meth.table.names._super)
-                    && sym != null && sym.owner.getQualifiedName() != meth.table.names.java_lang_Enum)
+            if (meth != null && (meth == meth.table.names._this || meth == meth.table.names._super))
                 tree.body.stats.tail = List.<JCStatement>nil();
             else
                 tree.body.stats = List.<JCStatement>nil();
@@ -272,7 +274,7 @@ public class Repair extends TreeTranslator {
         List<JCStatement> last = null;
         for (List<JCStatement> l = tree.stats; l.nonEmpty(); l = l.tail) {
             l.head = translate(l.head);
-            if (last == null && l.head.getTag() == JCTree.THROW)
+            if (last == null && l.head.hasTag(JCTree.Tag.THROW))
                 last = l;
         }
         if (last != null)
@@ -362,7 +364,7 @@ public class Repair extends TreeTranslator {
         List<JCStatement> last = null;
         for (List<JCStatement> l = tree.stats; l.nonEmpty(); l = l.tail) {
             l.head = translate(l.head);
-            if (last == null && l.head.getTag() == JCTree.THROW)
+            if (last == null && l.head.hasTag(JCTree.Tag.THROW))
                 last = l;
         }
         if (last != null)
@@ -370,16 +372,32 @@ public class Repair extends TreeTranslator {
         result = tree;
     }
 
+    @Override
+    public void visitLambda(JCTree.JCLambda tree) {        
+        if (!allowLambda) {
+            hasError = true;
+        }
+        super.visitLambda(tree);
+    }
+
+    @Override
+    public void visitReference(JCTree.JCMemberReference tree) {
+        if (!allowLambda) {
+            hasError = true;
+        }
+        super.visitReference(tree);
+    }
+
     @Override
     public void visitErroneous(JCErroneous tree) {
         hasError = true;
         result = tree;
     }
-    
+
     private JCStatement generateErrStat(DiagnosticPosition pos, String msg) {
         make.at(pos);
         ClassType ctype = (ClassType)syms.runtimeExceptionType;
-        Symbol ctor = rs.resolveConstructor(pos, attrEnv, ctype, List.of(syms.stringType), null, false, false);
+        Symbol ctor = rs.resolveConstructor(pos, attrEnv, ctype, List.of(syms.stringType), null);
         if (ctor.kind == Kinds.MTH) {
             JCLiteral literal = make.Literal(msg != null ? ERR_MESSAGE + " - " + msg : ERR_MESSAGE); //NOI18N
             JCNewClass tree = make.NewClass(null, null, make.QualIdent(ctype.tsym), List.<JCExpression>of(literal), null);
@@ -387,7 +405,7 @@ public class Repair extends TreeTranslator {
             tree.constructor = ctor;
             return make.Throw(tree);
         }
-        ctor = rs.resolveConstructor(pos, attrEnv, ctype, List.<Type>nil(), null, false, false);
+        ctor = rs.resolveConstructor(pos, attrEnv, ctype, List.<Type>nil(), null);
         if (ctor.kind == Kinds.MTH) {
             JCNewClass tree = make.NewClass(null, null, make.QualIdent(ctype.tsym), List.<JCExpression>nil(), null);
             tree.type = ctype;
@@ -413,12 +431,26 @@ public class Repair extends TreeTranslator {
         make.at(null);
         return make.MethodDef(sym, make.Block(0, List.<JCStatement>of(generateErrStat(null, null))));
     }
+    
+    private JCExpressionStatement generateFakeConstructorCall(MethodSymbol ctor) {
+        make.at(null);
+        JCTree.JCIdent ident = make.Ident(syms.objectType.tsym.name.table.names._this);
+        ident.sym = ctor;
+        ident.type = ctor.type;
+        List<JCExpression> args = List.nil();
+        for (Symbol.VarSymbol param : ((MethodSymbol)ctor).params().reverse()) {
+            args = args.prepend(make.Ident(param));
+        }
+        JCMethodInvocation meth = make.Apply(List.<JCExpression>nil(), ident, args);
+        meth.type = ctor.type.getReturnType();
+        return make.Exec(meth);
+    }
 
     private void translateClass(ClassSymbol c) {
         if (c == null)
             return;
         Type st = types.supertype(c.type);
-        if (st != null && st.tag == TypeTags.CLASS)
+        if (st != null && st.hasTag(TypeTag.CLASS))
             translateClass((ClassSymbol)st.tsym);
         LOGGER.finest("Repair.translateClass: " + c); //NOI18N
         if (repairedClasses.contains(c)) {
@@ -473,7 +505,7 @@ public class Repair extends TreeTranslator {
                             log.currentSource(),
                             tree,
                             "expected",         //NOI18N
-                            Token.IDENTIFIER);
+                            Tokens.TokenKind.IDENTIFIER);
                 } else if (fixedTopLevelName != null) {
                     c.fullname = Symbol.TypeSymbol.formFullName(c.name, enclosingElement);
                     c.flatname = Symbol.TypeSymbol.formFlatName(c.name, enclosingElement);
@@ -511,11 +543,11 @@ public class Repair extends TreeTranslator {
                     }
                     List<JCTree> last = null;
                     for (List<JCTree> l = tree.defs; l != null && l.nonEmpty(); l = l.tail) {
-                        if (l.head.getTag() == JCTree.METHODDEF)
+                        if (l.head.hasTag(JCTree.Tag.METHODDEF))
                             nonAbstractMethods.remove(((JCMethodDecl)l.head).sym);
                         hasError = false;
                         err = null;
-                        if (l.head.getTag() == JCTree.CLASSDEF && ((JCClassDecl)l.head).name == c.name.table.names.error) {
+                        if (l.head.hasTag(JCTree.Tag.CLASSDEF) && ((JCClassDecl)l.head).name == c.name.table.names.error) {
                             Scope.Entry prev = null;
                             for (Scope.Entry e = tree.sym.members_field.elems; e != null; e = e.sibling) {
                                 if (e.sym == ((JCClassDecl)l.head).sym) {
@@ -536,12 +568,17 @@ public class Repair extends TreeTranslator {
                                 tree.defs = l.tail;
                         } else {
                             l.head = translate(l.head);
-                            if ((l.head.getTag() == JCTree.METHODDEF && ((JCMethodDecl)l.head).sym == null)
-                                    || (l.head.getTag() == JCTree.VARDEF && ((JCVariableDecl)l.head).sym == null)) {
+                            if ((l.head.hasTag(JCTree.Tag.METHODDEF) && ((JCMethodDecl)l.head).sym == null)
+                                    || (l.head.hasTag(JCTree.Tag.VARDEF) && ((JCVariableDecl)l.head).sym == null)) {
                                 hasError = true;
+                            } else if (c.type != syms.objectType && l.head.hasTag(JCTree.Tag.METHODDEF)
+                                    && ((JCMethodDecl)l.head).body != null
+                                    && ((JCMethodDecl)l.head).name == ((JCMethodDecl)l.head).name.table.names.init
+                                    && TreeInfo.firstConstructorCall(l.head) == null) {
+                                ((JCMethodDecl)l.head).body.stats = ((JCMethodDecl)l.head).body.stats.append(generateFakeConstructorCall(((JCMethodDecl)l.head).sym));
                             }
                             if (hasError) {
-                                if (l.head.getTag() == JCTree.CLASSDEF && tree.sym.members_field.includes(((JCClassDecl)l.head).sym)) {
+                                if (l.head.hasTag(JCTree.Tag.CLASSDEF) && tree.sym.members_field.includes(((JCClassDecl)l.head).sym)) {
                                     last = l;
                                 } else {
                                     if (last != null)
