@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2006, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1999, 2012, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -71,20 +71,20 @@ public class Position {
      * @param   max         Number of characters to read
      * @param   expandTabs  If true, expand tabs when calculating columns
      */
-    public static LineMap makeLineMap(char[] src, int max, char replacedCharacter, boolean expandTabs) {
+    public static LineMap makeLineMap(char[] src, int max, boolean expandTabs) {
         LineMapImpl lineMap = expandTabs ?
-            new LineTabMapImpl(src.length) : new LineMapImpl();
-        lineMap.build(src, max, replacedCharacter);
+            new LineTabMapImpl(max) : new LineMapImpl();
+        lineMap.build(src, max);
         return lineMap;
     }
 
     /** Encode line and column numbers in an integer as:
-     *  line-number << LINESHIFT + column-number
-     *  {@link Position.NOPOS represents an undefined position.
+     *  {@code line-number << LINESHIFT + column-number }.
+     *  {@link Position#NOPOS} represents an undefined position.
      *
      * @param  line  number of line (first is 1)
      * @param  col   number of character on line (first is 1)
-     * @return       an encoded position or {@link Position.NOPOS
+     * @return       an encoded position or {@link Position#NOPOS}
      *               if the line or column number is too big to
      *               represent in the encoded format
      * @throws IllegalArgumentException if line or col is less than 1
@@ -107,8 +107,8 @@ public class Position {
          * @param line number of line (first is 1)
          * @return     position of first character in line
          * @throws  ArrayIndexOutOfBoundsException
-         *           if <tt>lineNumber < 1</tt>
-         *           if <tt>lineNumber > no. of lines</tt>
+         *           if {@code lineNumber < 1}
+         *           if {@code lineNumber > no. of lines}
          */
         int getStartPosition(int line);
 
@@ -119,8 +119,8 @@ public class Position {
          *
          * @return  position of character
          * @throws  ArrayIndexOutOfBoundsException
-         *           if <tt>line < 1</tt>
-         *           if <tt>line > no. of lines</tt>
+         *           if {@code line < 1}
+         *           if {@code line > no. of lines}
          */
         int getPosition(int line, int column);
 
@@ -147,16 +147,16 @@ public class Position {
 
         protected LineMapImpl() {}
 
-        public void build(char[] src, int max, char replacedCharacter) {
+        public void build(char[] src, int max) {
             int c = 0;
             int i = 0;
             int[] linebuf = new int[src.length + 1];
             linebuf[c++] = i;
             while (i < src.length) {
                 do {
-                    char ch = i == max ? replacedCharacter : src[i];
+                    char ch = src[i];
                     if (ch == '\r' || ch == '\n') {
-                        if (ch == '\r' && (i+1) < src.length && (i == max ? replacedCharacter : src[i+1]) == '\n')
+                        if (ch == '\r' && (i+1) < src.length && src[i+1] == '\n')
                             i += 2;
                         else
                             ++i;
